package iia.espacesEtats.graphes;

import iia.espacesEtats.modeles.Etat;

/**
 * un algorithme de recherche dans un espace d'tats, sous une forme arborescente.
 * Elle est "minimale" au sens où chaque noeud contient les information minimales,
 *  savoir :
 *   - l'tat associ  un noeud
 *   - un noeud parent (idalement, vers le "meilleur" parent connu, si cela a
 *     du sens pour l'algorithme de recherche considr.
 *      (null si l'tat correspond  un l'tat initial du problme).
 *
 * Cette classe pourra tre tendue si ncessaire, pour ajouter les informations
 * utiles en fonctin des besoins de chaque algorithme de recherche.
 *
 * @author Philippe Chatalic
 * 
 */
public class Noeud {

    // ------------------- Attributs -------------------
    /**
     * L'tat associ au noeud
     */
    private Etat etat;
    /**
     * le noeud correspondant  l'tat  partir duquel l'tat de ce noeud
     * a t produit lors de l'exploration du graphe de recherche
     */
    private Noeud pere;

    // ------------------- Constructeurs -------------------
    public Noeud() {
        etat = null;
        pere = null;
    }

    /**
     * Construit un noeud a partir
     */
    public Noeud(Etat e, Noeud n) {
        etat = e;
        pere = n;
    }

    // ------------------- Accesseurs -------------------
    /**
     * @return l'tat associ au noeud
     */
    public Etat getEtat() {
        return this.etat;
    }

    /**
     * @return le parent de ce noeud dans le graphe de recherche
     */
    public Noeud getPere() {
        return this.pere;
    }


    // ------------------- Modifieurs -------------------

    public void setPere(Noeud n) {
        pere = n;
    }

    public void setEtat(Etat e) {
        etat = e;
    }

    // ------------------- MoAutres mthodesdifieurs -------------------


 
    /**
     * @return vrai si le noeud n correspond au mme tat que le noeud courant
     */
    public boolean memeEtat(Noeud n) {
        return etat.equals(n.getEtat());
    }
    
    public boolean memeEtat(Etat e) {
        return e.equals(this.getEtat());
    }

    public String toString() {
        String ret = "Noeud : etat = " + etat;
        return ret;
    }



}
